/* This test driver program encodes a data record and writes the        */
/* encoded result to an output file..                                   */

#include <stdio.h>
#include <stdlib.h>
#include "rtxsrc/rtxXmlStr.h"
#include "rtxsrc/rtxStreamSocket.h"
#include "employee.h"

#define DEFAULT_URL "http://127.0.0.1:9124"
#define MAX_CONNECT_ATTEMPTS    3

static void fill_jSmith (OSCTXT* pctxt, PersonnelRecord* pPR);

int main (int argc, char** argv)
{
   PersonnelRecord employee;
   OSCTXT	ctxt;
   int		i, numMsgs=2, stat, port = 0;
   const char*  url = DEFAULT_URL;
   const char*  filename = 0;
   char buf[100];
   OSBOOL       trace = TRUE, verbose = FALSE;
   OSRTSOCKET   socket;

   /* Process command line arguments */

   if (argc > 1) {
      for (i = 1; i < argc; i++) {
         if (!strcmp (argv[i], "-v")) verbose = TRUE;
         else if (!strcmp (argv[i], "-n")) numMsgs = atoi (argv[++i]);
         else if (!strcmp (argv[i], "-notrace")) trace = FALSE;
         else if (!strcmp (argv[i], "-f")) filename = argv[++i];
         else if (!strcmp (argv[i], "-url")) {
            url = argv[++i];
         }
         else {
            printf ("usage: writer [-v] [-url <url>] [-notrace]\n");
            printf ("   -v         verbose mode: print trace info\n");
            printf ("   -url       bind URL http://host:port\n");
            printf ("   -n         number of messages to send\n");
            printf ("   -f <file>  send encoded file <file> to a stream\n");
            printf ("   -notrace  do not display trace info\n");
            return 0;
         }
      }
   }

   {
      char* portStr;

      strcpy (buf, url);
      portStr = strstr (buf, "://");
      if (portStr != 0) {
         url = portStr + 3;
      }
      else
         url = buf;

      portStr = strchr (url, ':');
      if (portStr != 0) {
         *portStr = 0;
         portStr++;
         port = atoi (portStr);
      }
   }

   /* Init context */

   stat = rtXmlInitContext (&ctxt);
   if (0 != stat) {
      printf ("Context initialization failed.\n");
      rtxErrPrint (&ctxt);
      return stat;
   }

   if (verbose) rtxSetDiag (&ctxt, TRUE);

   /* One-time socket package initialization (required for Windows only) */

   stat = rtxSocketsInit();
   if (0 != stat) {
      printf ("Socket initialization failed.\n");
      rtxErrSetData (&ctxt, stat, __FILE__, __LINE__);
      rtxErrPrint (&ctxt);
      return stat;
   }

   stat = rtxSocketCreate (&socket);
   if (0 != stat) {
      printf ("Socket creation failed.\n");
      rtxErrSetData (&ctxt, stat, __FILE__, __LINE__);
      rtxErrPrint (&ctxt);
      return stat;
   }

   if (trace) printf ("Connecting to %s:%i...", url, port);
   for (i = 0; i < MAX_CONNECT_ATTEMPTS; i++) {
      stat = rtxSocketConnect (socket, url, port);
      if (0 != stat) {
         if (i + 1 == MAX_CONNECT_ATTEMPTS) {
            printf ("\nConnection failed, error code is %i\n", stat);
            return stat;
         }
      }
      else break;
   }
   if (trace) printf ("\nConnection established.\n");

   stat = rtxStreamSocketAttach 
      (&ctxt, socket, OSRTSTRMF_OUTPUT);

   if (0 != stat) {
      printf ("Stream initialization failed.\n");
      rtxErrSetData (&ctxt, stat, __FILE__, __LINE__);
      rtxErrPrint (&ctxt);
      return stat;
   }
   printf ("sending %d messages..\n", numMsgs);

   for (i = 0; i < numMsgs; i++) {
      if (filename == 0) {
         /* Populate structure of generated type */

         Init_PersonnelRecord (&ctxt, &employee);

         fill_jSmith (&ctxt, &employee);

         /* Encode */

         stat = XmlE_personnelRecord (&ctxt, &employee);

         if (trace) {
            if (0 == stat) {
               printf ("Encoding is successful.\n");
            }
            else {
               printf ("Encoding failed\n");
               rtxErrPrint (&ctxt);
               return stat;
            }
         }
      }
      else {
         FILE* fp;

         /* Read input file into a memory buffer */

         if ((fp = fopen (filename, "rb")) != 0) {
            long len;
            OSOCTET* msgbuf;

            /* get the length of file */
            
            fseek (fp, 0, SEEK_END);
            len = ftell (fp);
            fseek (fp, 0, SEEK_SET);

            msgbuf = rtxMemAlloc (&ctxt, len);
            if (fread (msgbuf, 1, len, fp) != (size_t)len) {
               printf ("Error reading %s.\n", filename);
               return -1;
            }
            fclose (fp);   
            stat = rtxSocketSend (socket, msgbuf, (int)len);
            if (trace) {
               if (stat == 0) {
                  printf ("Sending is successful.\n");
               }
               else {
                  printf ("Sending is failed.\n");
                  rtxErrSetData (&ctxt, stat, __FILE__, __LINE__);
                  rtxErrPrint (&ctxt);
                  return stat;
               }
            }      

            rtxMemFreePtr (&ctxt, msgbuf);
         }
         else {
            printf ("Error opening %s for read access\n", filename);
            return -1;
         }
      }
   }
   rtxStreamClose (&ctxt);
   rtxFreeContext (&ctxt);
   return 0;
}

/* Routine to fill the employee record with test data */

static void fill_Name 
(Name* pName, const char* fn, const char* mi, const char* ln);

static void fill_jSmith (OSCTXT* pctxt, PersonnelRecord* pPR)
{
   ChildInformation* pChildInfo;

   fill_Name (&pPR->name, "John", "P", "Smith");

   pPR->title.value = (const OSUTF8CHAR*) "Director";
   pPR->number = 51;
   pPR->dateOfHire.value = (const OSUTF8CHAR*) "19710917";

   fill_Name (&pPR->nameOfSpouse, "Mary", "T", "Smith");

   rtxDListInit (&pPR->children._elemList);

   /* fill first child record */

   pChildInfo = OSRTALLOCTYPE (pctxt, ChildInformation);
   Init_ChildInformation (pctxt, pChildInfo);
   fill_Name (&pChildInfo->name, "Ralph", "T", "Smith");
   pChildInfo->dateOfBirth.value = (const OSUTF8CHAR*) "19571111";
   rtxDListAppend (pctxt, &pPR->children._elemList, pChildInfo);

   /* fill second child record */

   pChildInfo = OSRTALLOCTYPE (pctxt, ChildInformation);
   Init_ChildInformation (pctxt, pChildInfo);
   fill_Name (&pChildInfo->name, "Susan", "B", "Jones");
   pChildInfo->dateOfBirth.value = (const OSUTF8CHAR*) "19590717";
   rtxDListAppend (pctxt, &pPR->children._elemList, pChildInfo);
}

/* Routine to fill a name structure */

static void fill_Name 
(Name* pName, const char* fn, const char* mi, const char* ln)
{
   pName->givenName.value  = (const OSUTF8CHAR*) fn;
   pName->initial.value    = (const OSUTF8CHAR*) mi;
   pName->familyName.value = (const OSUTF8CHAR*) ln;
}

