/*
 * Copyright (c) 2017 Objective Systems, Inc.
 *
 * See the COPYING.txt file in this subdirectory for conditions on use
 * of this software.  This notice applies to software only in this
 * subdirectory (oortsrc) and not to other software that may be a
 * a part of this distribution.
 *
 *****************************************************************************/
/**
 * @file ooConnEnd.h
 * Connection endpoint structures and functions.
 */
#ifndef _OOCONNEND_H_
#define _OOCONNEND_H_

#include "oortsrc/ooLinkedList.h"
#include "oortsrc/ooSocket.h"

struct _OOConnEnd;

typedef int (*OOSockConnCbFunc)(struct _OOConnEnd* pCE);
typedef int (*OOSockReadCbFunc)(struct _OOConnEnd* pCE);
typedef int (*OOSockDisconCbFunc)(struct _OOConnEnd* pCE);

typedef struct _OOConnEnd {
   OOSOCKET     socket;
   OOLinkedList outBufList;
   OOBOOL       connected;

   /* Callback functions */
   OOSockConnCbFunc connectCB;
   OOSockReadCbFunc readCB;
   OOSockDisconCbFunc disconCB;
} OOConnEnd;

#ifdef __cplusplus
extern "C" {
#endif
/**
 * This function accepts a connection request on the given endpoint.
 * It creates a new endpoint for communications with the peer socket.
 * It then invokes the connect callback with the newly created endpoint
 * object as a parameter.
 *
 * @param pCE          Pointer to acceptor connection endpoint object.
 * @return             Status of accept operation.
 */
EXTERN int ooConnEndAccept (OOConnEnd* pCE);

/**
 * This function creates and initializes a new connection endpoint
 * object.  This version uses an existing socket handle.  The socket
 * must have been set to non-blocking I/O mode.
 *
 * @param socket       Socket handle.
 * @return             Point to connection endpoint object.
 */
EXTERN OOConnEnd* ooConnEndCreate (OOSOCKET socket);

/**
 * This function creates and initializes a new connection endpoint
 * object.  This version creates a new socket handle.
 *
 * @return             Point to connection endpoint object or NULL if
 *                     socket creation was not successful.
 */
EXTERN OOConnEnd* ooConnEndCreateNew (void);

/**
 * This function deletes a connection endpoint object.
 *
 * @param pCE          Pointer to connection endpoint object.
 */
EXTERN void ooConnEndDelete (OOConnEnd* pCE);

/**
 * This function listens for new incoming connection requests.
 *
 * @param pCE           Pointer to connection endpoint object.
 * @param port          Port on which to listen for connections.
 * @param connectCB     Connection callback function.
 * @return              Status of listen request
 *                        (0 = OK, negative value = error).
 */
EXTERN int ooConnEndListen
(OOConnEnd* pCE, int port, OOSockConnCbFunc connectCB);

/**
 * This function looks up a connection endpoint object using its socket
 * handle as the key.
 *
 * @param socket  Socket handle.
 * @return  Pointer to connection endpoint object or NULL if not found.
 */
EXTERN OOConnEnd* ooConnEndLookupUsingSocket (OOSOCKET socket);

/**
 * This function executes a continuous loop to monitor endpoints and
 * execute event handlers when data is detected.
 */
EXTERN void ooConnEndReadLoop (void);

/**
 * This function writes data to the given connection endpoint.
 *
 * @param pConnEnd  Pointer to connection endpoint object.
 * @param pdata     Pointer to data to be written.
 * @param length    Length of data in bytes to be written
 * @param dynamic   Is pdata buffer dynamic?
 * @return          Status of write operation
 */
EXTERN int ooConnEndWrite
(OOConnEnd* pConnEnd, OOUINT8* pdata, OOUINT32 length, OOBOOL dynamic);

/**
 * This function writes data to that is queued for output to the given
 * connection endpoint.
 *
 * @param pConnEnd  Pointer to connection endpoint object.
 * @return          Status of write operation
 */
EXTERN int ooConnEndWriteQ (OOConnEnd* pConnEnd);

#ifdef __cplusplus
}
#endif
#endif
