/*
 * Copyright (c) 2017 Objective Systems, Inc.
 *
 * See the COPYING.txt file in this subdirectory for conditions on use
 * of this software.  This notice applies to software only in this
 * subdirectory (oortsrc) and not to other software that may be a
 * a part of this distribution.
 *
 *****************************************************************************/
/**
 * @file ooSocket.h
 * Common runtime constants, data structure definitions, and run-time functions
 * to support the socket operations.
 */
#ifndef _OOSOCKET_H_
#define _OOSOCKET_H_

#include "oortsrc/ooCommon.h"

#ifdef __cplusplus
extern "C" {
#endif

/**
 * @defgroup sockets Socket Layer
 * @{
 */
#if defined (_WIN64)
typedef unsigned __int64 OOSOCKET;      /**< Socket handle */
#elif defined (_WIN32)
typedef unsigned int OOSOCKET;          /**< Socket handle */
#else
typedef int OOSOCKET;                   /**< Socket handle */
#endif

#define OOSOCKET_INVALID ((OOSOCKET)-1)

/**
 * The IP address represented as unsigned long value. The most significant 8
 * bits in this unsigned long value represent the first number of the IP
 * address. The least significant 8 bits represent the last number of the IP
 * address.
 */
typedef unsigned long OOIPADDR;

#define OOIPADDR_ANY   ((OOIPADDR)0)
#define OOIPADDR_LOCAL ((OOIPADDR)0x7f000001UL) /* 127.0.0.1 */

/* Socket informational return codes */

#define OOINCOMPLETE            1       /* I/O operation not complete */
#define OODISCONNECT            2       /* Peer disconnected          */

/* Socket error codes (-100 to -200) */

#define OOERRSOCKNOTINIT        (-100)
#define OOERRSOCKINVALID        (-101)
#define OOERRSOCKSETOPT         (-102)
#define OOERRSOCKCLOSE          (-103)
#define OOERRSOCKBIND           (-104)
#define OOERRSOCKLISTEN         (-105)
#define OOERRSOCKACCEPT         (-106)
#define OOERRSOCKCONNECT        (-107)
#define OOERRSOCKWRITE          (-108)
#define OOERRSOCKREAD           (-109)
#define OOERRSOCKGETHOST        (-110)
#define OOERRSOCKTIMEOUT        (-111)
#define OOERRSOCKSELECT         (-112)

/**
 * This function permits an incoming connection attempt on a socket. It
 * extracts the first connection on the queue of pending connections on socket.
 * It then creates a new socket and returns a handle to the new socket. The
 * newly created socket is the socket that will handle the actual connection
 * and has the same properties as original socket. See description of 'accept'
 * socket function for further details.
 *
 * @param socket       Socket handle created by ooSocketCreate
 * @param pNewSocket   The pointer to variable to receive new socket handle
 * @param destAddr     Optional pointer to a buffer that receives the IP
 *                     address of the connecting entity. It may be NULL.
 * @param destPort     Optional pointer to a buffer that receives the port of
 *                     the connecting entity. It may be NULL.
 * @return             Completion status of operation: 0 = success,
 *                     negative return value is error.
 */
EXTERN int ooSocketAccept (OOSOCKET socket, OOSOCKET *pNewSocket,
                           OOIPADDR* destAddr, int* destPort);

/**
 * This function converts an IP address to its string representation.
 *
 * @param ipAddr       The IP address to be converted.
 * @param pbuf         Pointer to the buffer to receive a string with the IP
 *                     address.
 * @param bufsize      Size of the buffer.
 * @return             Completion status of operation: 0 = success,
 *                     negative return value is error.
 */
EXTERN int ooSocketAddrToStr (OOIPADDR ipAddr, char* pbuf, int bufsize);

/**
 * This function associates a local address with a socket. It is used on an
 * unconnected socket before subsequent calls to the ::rtSocketConnect or
 * ::rtSocketListen functions. See description of 'bind' socket function for
 * further details.
 *
 * @param socket       Socket handle created by ooSocketCreate
 * @param addr         The local IP address to assign to the socket.
 * @param port         The local port number to assign to the socket.
 * @return             Completion status of operation: 0 = success,
 *                     negative return value is error.
 */
EXTERN int ooSocketBind (OOSOCKET socket, OOIPADDR addr, int port);

/**
 * This function closes an existing socket.
 *
 * @param socket       Socket handle
 * @return             Completion status of operation: 0 = success,
 *                     negative return value is error.
 */
EXTERN int ooSocketClose (OOSOCKET socket);

/**
 * This function establishes a connection to a specified socket. It is used to
 * create a connection to the specified destination. When the socket call
 * completes successfully, the socket is ready to send and receive data. See
 * description of 'connect' socket function for further details.
 *
 * @param socket       Socket handle created by ooSocketCreate
 * @param host         The null-terminated string with the IP address in the
 *                     following format: "NNN.NNN.NNN.NNN", where NNN is a
 *                     number in the range (0..255).
 * @param port         The destination port to connect.
 * @return             Completion status of operation: 0 = success,
 *                     negative return value is error.
 */
EXTERN int ooSocketConnect (OOSOCKET socket, const char* host, int port);

/**
 * This function creates a socket. The only streaming TCP/IP sockets are
 * supported at the moment.
 *
 * @param psocket      The pointer to the socket's handle variable to receive
 *                     the handle of new socket.
 * @return             Completion status of operation: 0 = success,
 *                     negative return value is error.
 */
EXTERN int ooSocketCreate (OOSOCKET* psocket);

/**
 * This function creates an asynchronous socket. This is a socket set to
 * non-blocking mode so that all I/O requests are queued and then control
 * is returned to the caller.
 *
 * @param psocket      Pointer to socket handle variable to receive
 *                     the handle of new socket.
 * @return             Completion status of operation: 0 = success,
 *                     negative return value is error.
 */
EXTERN int ooSocketCreateAsync (OOSOCKET* psocket);

/**
 * This function creates a UDP datagram socket.
 *
 * @param psocket      Pointer to socket handle variable to receive
 *                     the handle of new socket.
 * @return             Completion status of operation: 0 = success,
 *                     negative return value is error.
 */
EXTERN int ooSocketCreateUDP (OOSOCKET* psocket);

/**
 * This function initiates use of sockets by an application. This function must
 * be called first before use sockets.
 *
 * @return             Completion status of operation: 0 = success,
 *                     negative return value is error.
 */
EXTERN int ooSocketsInit (void);

/**
 * This function terminates use of sockets by an application. This function
 * must be called after done with sockets.
 *
 * @return             Completion status of operation: 0 = success,
 *                     negative return value is error.
 */
EXTERN int ooSocketsCleanup (void);

/**
 * This function places a socket a state where it is listening for an incoming
 * connection. To accept connections, a socket is first created with the
 * ::rtSocketCreate function and bound to a local address with the
 * ::rtSocketBind function, a maxConnection for incoming connections is
 * specified with ::rtSocketListen, and then the connections are accepted with
 * the ::rtSocketAccept function. See description of 'listen' socket function
 * for further details.
 *
 * @param socket       Socket handle created by ooSocketCreate
 * @param maxConnection Maximum length of the queue of pending connections.
 * @return              Completion status of operation: 0 =
 *                      success, negative return value is error.
 */
EXTERN int ooSocketListen (OOSOCKET socket, int maxConnection);

/**
 * This function receives data from a connected socket. The socket must
 * be connected before calling this function. See description of the 'recv'
 * system socket function for further details.
 *
 * @param socket       Socket handle
 * @param pbuf         Pointer to the buffer for the incoming data.
 * @param bufsize      Length of the buffer.
 * @return             If no error occurs, returns the number of bytes
 *                     received. Otherwise, negative value is error code.
 */
EXTERN int ooSocketRecv (OOSOCKET socket, OOUINT8* pbuf, OOUINT32 bufsize);

/**
 * This function receives data from a connected/unconnected socket. It is used
 * to read incoming data on sockets. It populates the remotehost and
 * remoteport parameters with information of remote host. See description of
 * 'recvfrom' socket function for further details.
 *
 * @param socket       Socket handle created by call to ooSocketCreate.
 * @param pbuf         Pointer to the buffer for the incoming data.
 * @param bufsize      Length of the buffer.
 * @param remotehost   Pointer to a buffer in which remote ip address
 *                     will be returned.
 * @param remoteport   Pointer to an int in which remote port number
 *                     will be returned.
 *
 * @return             If no error occurs, returns the number of bytes
 *                     received. Otherwise, the negative value is error code.
 */
EXTERN int ooSocketRecvFrom (OOSOCKET socket, OOUINT8* pbuf,
                             OOUINT32 bufsize, char* remotehost,
                             int* remoteport);
/**
 * This function receives data from a connected socket. The socket must
 * be connected before calling this function.  It also allows a timeout
 * value in seconds to be provided. If the I/O does not complete within
 * the given time, an OOERRSOCKTIMEOUT status is returned.
 *
 * @param socket       Socket handle
 * @param pbuf         Pointer to the buffer for the incoming data.
 * @param bufsize      Length of the buffer.
 * @param nsecs        Number of seconds to wait for recv to complete.
 * @return             If no error occurs, returns the number of bytes
 *                     received. Otherwise, negative value is error code.
 */
EXTERN int ooSocketRecvTimed
(OOSOCKET socket, OOUINT8* pbuf, OOUINT32 bufsize, OOUINT32 nsecs);

/**
 * This function sends data on a connected socket. This function will
 * block until either all of the data is written to the socket or an
 * error occurs.
 *
 * @param socket       Socket handle created by ooSocketCreate
 * @param pdata        Buffer containing the data to be transmitted.
 * @param size         Length of the data in pdata.
 * @return             Completion status of operation: 0 = success,
 *                     negative return value is error.
 */
EXTERN int ooSocketSend
(OOSOCKET socket, const OOUINT8* pdata, OOUINT32 size);

/**
 * This function sends data asynchronously to a connected socket. This
 * will make a single call to the base socket write function.  If the
 * I/O does not complete immediately, internal offsets in the message
 * buffer are updated and OOINCOMPLETE is returned.  This function can
 * then be called again to complete the I/O.
 *
 * @param socket       Socket handle created by ooSocketCreate
 * @param pbuffer      Message buffer describing message to be sent
 * @return             Completion status of operation: 0 = success,
 *                     OOINCOMPLETE = OK but I/O did not complete,
 *                     negative return value is error.
 */
EXTERN int ooSocketSendAsync (OOSOCKET socket, OOMsgBuf* pbuffer);

/**
 * This function sends data on a connected socket. This function will
 * block until either all of the data is written to the socket or an
 * error occurs. It also allows a timeout value in seconds to be provided.
 * If the I/O does not complete within the given time, an OOERRSOCKTIMEOUT
 * status is returned.
 *
 * @param socket       Socket handle created by ooSocketCreate
 * @param pdata        Buffer containing the data to be transmitted.
 * @param size         Length of the data in pdata.
 * @param nsecs        Number of seconds to wait for I/O to complete.
 * @return             Completion status of operation: 0 = success,
 *                     negative return value is error.
 */
EXTERN int ooSocketSendTimed
(OOSOCKET socket, const OOUINT8* pdata, OOUINT32 size, OOUINT32 nsecs);

/**
 * This function sends data on a connected or unconnected socket. See
 * description of 'sendto' socket function for further details.
 *
 * @param socket       Socket handle created by ooSocketCreate
 * @param pdata        Buffer containing the data to be transmitted.
 * @param size         Length of the data in pdata.
 * @param remotehost   Remote host ip address to which data has to
 *                     be sent.
 * @param remoteport   Remote port ip address to which data has to
 *                     be sent.
 *
 * @return             Completion status of operation: 0 = success,
 *                     negative return value is error.
 */
EXTERN int ooSocketSendTo (OOSOCKET socket, const OOUINT8* pdata,
                           OOUINT32 size, const char* remotehost,
                           int remoteport);

/**
 * This function is used for synchronous monitoring of multiple sockets.
 * For more information refer to documnetation of "select" system call.
 *
 * @param nfds         The highest numbered descriptor to be monitored
 *                     plus one.
 * @param readfds      The descriptors listed in readfds will be watched for
 *                     whether read would block on them.
 * @param writefds     The descriptors listed in writefds will be watched for
 *                     whether write would block on them.
 * @param exceptfds    The descriptors listed in exceptfds will be watched for
 *                     exceptions.
 * @param timeout      Upper bound on amout of time elapsed before select
 *                     returns.
 * @return             Completion status of operation: 0 = success,
 *                     negative return value is error.
 */
EXTERN int ooSocketSelect (int nfds, fd_set *readfds, fd_set *writefds,
                           fd_set *exceptfds, struct timeval * timeout);

/**
 * This function converts the string with IP address to a double word
 * representation. The converted address may be used with the ::ooSocketBind
 * function.
 *
 * @param pIPAddrStr   The null-terminated string with the IP address in the
 *                     following format: "NNN.NNN.NNN.NNN", where NNN is a
 *                     number in the range (0..255).
 *
 * @param pIPAddr      Pointer to the converted IP address. This parameter
 *                     may be NULL if the function is only to be used for
 *                     validation of the string format.
 *
 * @return             Completion status of operation: 0 = success,
 *                     negative return value is error.
 */
EXTERN int ooSocketStrToAddr (const char* pIPAddrStr, OOIPADDR* pIPAddr);

/**
 * This function converts the string with IP address and port to a numeric
 * IP address and port value. The converted address may be used with the
 * ::ooSocketBind function.
 *
 * @param pIPAddrStr   The null-terminated string with IP address and port
 *                     in the following format: "NNN.NNN.NNN.NNN[:P+]",
 *                     where NNN is a number in the range (0..255) and
 *                     P+ is one or more digits representing the port
 *                     number.  The port portion of the string is optional.
 *
 * @param pIPAddr      Pointer to the converted IP address.  This parameter
 *                     may be NULL if the function is only to be used for
 *                     validation of the string format.
 *
 * @param pPort        Pointer to integer to receive converted port number.
 *                     This parameter is optional; NULL can be passed if
 *                     the function is to be used for validation of the
 *                     string format only.
 *
 * @return             Completion status of operation: 0 = success,
 *                     negative return value is error.
 */
EXTERN int ooSocketStrToAddrAndPort
(const char* pIPAddrStr, OOIPADDR* pIPAddr, int* pPort);

/**
 * This function retrives the IP address of the local host.
 *
 * @param pIPAddrs   Pointer to a char buffer in which local IP address will be
 *                   returned.
 * @return           Completion status of operation: 0 = success,
 *                   negative return value is error.
 */
EXTERN int ooGetLocalIPAddress (char * pIPAddrs);

EXTERN long ooHTONL (long val);

EXTERN short ooHTONS (short val);
/**
 * @}
 */
#ifdef __cplusplus
}
#endif

#endif /* _OOSOCKET_H_ */
