/*
 * Copyright (c) 2003-2018 Objective Systems, Inc.
 *
 * This software is furnished under a license and may be used and copied
 * only in accordance with the terms of such license and with the
 * inclusion of the above copyright notice. This software or any other
 * copies thereof may not be provided or otherwise made available to any
 * other person. No title to and ownership of the software is hereby
 * transferred.
 *
 * The information in this software is subject to change without notice
 * and should not be construed as a commitment by Objective Systems, Inc.
 *
 * PROPRIETARY NOTICE
 *
 * This software is an unpublished work subject to a confidentiality agreement
 * and is protected by copyright and trade secret law.  Unauthorized copying,
 * redistribution or other use of this work is prohibited.
 *
 * The above notice of copyright on this source code product does not indicate
 * any actual or intended publication of such source code.
 *
 *****************************************************************************/

#ifndef __DOMAPI_H
#define __DOMAPI_H

#include "rtxsrc/rtxCommon.h"
#include "rtdomsrc/rtxDomDefs.h"
#include "rtxmlsrc/osrtxml.h"
#include "rtxmlsrc/rtXmlNamespace.h"

/**
 * @defgroup dom DOM API functions.
 * @{
 */
#ifdef __cplusplus
extern "C" {
#endif

typedef void* OSRTDOMDocPtr;
typedef void* OSRTDOMNodePtr;
typedef void* OSRTDOMAttrPtr;
typedef void* OSRTDOMNsNodePtr;
typedef void* OSRTDOMContCtxtPtr;
typedef void* OSRTDOMAttrCtxtPtr;

/** 0  - OK
    <0 - Error code (see rtxsrc/rtxErrCodes.h)
    >0 - Supplementary code
*/
typedef int OSRTDOMError;

/**
 * Inits parser.
 */
EXTERNDOM void domParserInit ();

/**
 * Shutdowns parser
 */
EXTERNDOM void domParserShutdown ();

/**
 * Parse the file into a DOM document.
 *
 * @param fileSpec  File name
 * @param pXmlDoc   Pointer to a pointer to newly created DOM document.
 * @return   0 - success, otherwise error code.
 */
EXTERNDOM OSRTDOMError domParseFile (const char* fileSpec, OSRTDOMDocPtr* pXmlDoc);

/**
 * Returns root node for the document.
 *
 * @param xmlDoc     Pointer to the DOM document context
 * @param pNode      Pointer to pointer to receive the root node.
 * @return   0 - success, otherwise error code.
 */
EXTERNDOM OSRTDOMError domGetRootElement (OSRTDOMDocPtr xmlDoc, OSRTDOMNodePtr* pNode);

/**
 * Returns the document for the given element.
 *
 * @param node       Pointer to a node.
 * @param pXmlDoc    Pointer to a pointer to the DOM document context
 * @return   0 - success, otherwise error code.
 */
EXTERNDOM OSRTDOMError domGetDoc (OSRTDOMNodePtr node, OSRTDOMDocPtr* pXmlDoc);

/**
 * Frees document.
 * @param xmlDoc     Pointer to the DOM document context
 */
EXTERNDOM void domFreeDoc (OSRTDOMDocPtr xmlDoc);

/**
 * Returns next element node (down sibling).
 *
 * @param curNode    Pointer to current node,
 * @param pNextNode  Pointer to pointer to receive the next node.
 * @return   0 - success, otherwise error code.
 */
EXTERNDOM OSRTDOMError domGetNext (const OSRTDOMNodePtr curNode,
                                   OSRTDOMNodePtr* pNextNode);

/**
 * Returns first (top) child node.
 *
 * @param curNode    Pointer to current node,
 * @param pChildNode Pointer to pointer to receive the child node.
 * @return   0 - success, otherwise error code.
 */
EXTERNDOM OSRTDOMError domGetChild (const OSRTDOMNodePtr curNode,
                                    OSRTDOMNodePtr* pChildNode);

/**
 * Returns node's name and prefix (if any).
 *
 * @param curNode    Pointer to current node,
 * @param ppName     Pointer to pointer to receive the node's local name.
 * @param ppNsPrefix Pointer to pointer to receive the node's prefix.
 *                     May be NULL, if prefix is not important.
 * @param ppNsUri    Pointer to pointer to receive the namespace's Uri.
 * @return   0 - success, otherwise error code.
 */
EXTERNDOM OSRTDOMError domGetElementName (const OSRTDOMNodePtr curNode,
                                          const OSUTF8CHAR** ppName,
                                          const OSUTF8CHAR** ppNsPrefix,
                                          const OSUTF8CHAR** ppNsUri);

/**
 * Gets the node's content.
 *
 * @param ppCtxt     Pointer to an implementation-specific content context pointer;
 * @param curNode    Pointer to the current node;
 * @param ppValue    Pointer to pointer to receive value;
 * @param pValueLen  Pointer to value length; may be NULL.
 * @param pCdataProcessed  Pointer to boolean value; this value will be set to
 *                     TRUE if CDATA section was proceeded; otherwise FALSE.
 * @return  0  - if content retrieved successfully
 *         <0 - the error code;
 */
EXTERNDOM int domGetNodeContent (OSRTDOMContCtxtPtr* ppCtxt,
                                 const OSRTDOMNodePtr curNode,
                                 const OSUTF8CHAR** ppValue,
                                 size_t* pValueLen,
                                 OSBOOL* pCdataProcessed);

/**
 * Returns the pointer to the first (top) attribute of the node.
 * The context pointer will be passed to subsequent calls
 * to domGetNextAttr.
 *
 * @param ppCtxt     Pointer to an implementation-specific context pointer,
 * @param curNode    Pointer to current node,
 * @param pTopAttrNode Pointer to pointer to receive the first
 *                     attribute node. domGetNextAttr/domGetPrevAttr
 *                     functions may be used to get next/previous
 *                     attributes.
 * @return   0 - success, otherwise error code.
 */
EXTERNDOM OSRTDOMError domGetNodeFirstAttribute (OSRTDOMAttrCtxtPtr* ppCtxt,
                                                 const OSRTDOMNodePtr curNode,
                                                 OSRTDOMAttrPtr* pTopAttrNode);

/**
 * Returns number of attributes in the node.
 *
 * @param curNode    Pointer to current node,
 * @return           Number of attributes in the node.
 */
EXTERNDOM int domGetNodeAttributesNum (const OSRTDOMNodePtr curNode);

/**
 * Returns the next attribute.
 *
 * @param ppCtxt       Pointer to an implementation-specific context pointer,
 * @param curAttrNode  Pointer to current attribute node,
 * @param pAttrNode    Pointer to pointer to receive the next
 *                       attribute node.
 * @return   0 - success, otherwise error code.
 */
EXTERNDOM OSRTDOMError domGetNextAttr (OSRTDOMAttrCtxtPtr* ppCtxt,
                                       const OSRTDOMAttrPtr curAttrNode,
                                       OSRTDOMAttrPtr* pAttrNode);
/**
 * Returns the attribute's name and value.
 *
 * @param curAttrNode  Pointer to current attribute node,
 * @param ppAttrName   Pointer to pointer to receive attribute's name.
 * @param ppAttrValue  Pointer to pointer to receive attribute's value.
 * @param ppAttrPrefix Pointer to pointer to receive namespace's prefix.
 * @param ppAttrUri    Pointer to pointer to receive namespace's uri.
 * @return   0 - success, otherwise error code.
 */
EXTERNDOM OSRTDOMError domGetAttrData (const OSRTDOMAttrPtr curAttrNode,
                                       const OSUTF8CHAR** ppAttrName,
                                       const OSUTF8CHAR** ppAttrValue,
                                       const OSUTF8CHAR** ppAttrPrefix,
                                       const OSUTF8CHAR** ppAttrUri);

/* Encode */
/**
 * Saves DOM document to a file. This function is supplementary and
 * it is not obligatory to implement it.
 *
 * @param xmlDoc     Pointer to the DOM document context.
 * @param filename   Pointer to file name.
 * @return   0 - success, otherwise error code.
 */
EXTERNDOM OSRTDOMError domSaveDoc (OSRTDOMDocPtr xmlDoc,
                                   const char* filename);

/**
 * Creates a new document and a root element.
 *
 * @param pctxt        Pointer to context block structure.
 * @param pXmlDoc  Pointer to pointer to the newly created empty
 *                   DOM document.
 * @param pNodeName  Name of the node
 * @param pNS          XML namespace information (prefix and URI).
 * @param pNSAttrs     List of namespace attributes to be added to element.
 * @return   0 - success, otherwise error code.
 */
EXTERNDOM OSRTDOMError domCreateDocument (OSCTXT* pctxt,
                                          OSRTDOMDocPtr* pXmlDoc,
                                          const OSUTF8CHAR* pNodeName,
                                          OSXMLNamespace* pNS,
                                          OSRTDList* pNSAttrs);

/**
 * Creates new child node.
 *
 * @param pctxt      Pointer to context block structure.
 * @param parentNode Pointer to the parent node
 * @param pNodeName  Name of the node
 * @param pNS          XML namespace information (prefix and URI).
 * @param pNSAttrs     List of namespace attributes to be added to element.
 * @param pNewNode   Pointer to pointer to newly created node.
 * @return   0 - success, otherwise error code.
 */
EXTERNDOM OSRTDOMError domCreateChild (OSCTXT* pctxt,
                                       OSRTDOMNodePtr parentNode,
                                       const OSUTF8CHAR* pNodeName,
                                       OSXMLNamespace* pNS,
                                       OSRTDList* pNSAttrs,
                                       OSRTDOMNodePtr* pNewNode);

/**
 * Creates new attribute and adds it to the node.
 *
 * @param pctxt      Pointer to context block structure.
 * @param node       Pointer to the node where attribute to be added.
 * @param pAttrName  Pointer to the null-terminated string containing
 *                     name of the attribute.
 * @param pAttrValue Pointer to the null-terminated string containing
 *                     name of the value.
 * @param pNS          XML namespace information (prefix and URI).
 * @param pNSAttrs     List of namespace attributes to be added to element.
 * @return   0 - success, otherwise error code.
 */
EXTERNDOM OSRTDOMError domAddAttribute (OSCTXT* pctxt,
                                        OSRTDOMNodePtr node,
                                        const OSUTF8CHAR* pAttrName,
                                        const OSUTF8CHAR* pAttrValue,
                                        OSXMLNamespace* pNS,
                                        OSRTDList* pNSAttrs);

/**
 * Adds content (text) to the node.
 *
 * @param node       Pointer to the node where content to be added.
 * @param pContent   Pointer to UTF-8 encoded content.
 * @param contentLen Length of the content
 * @return   0 - success, otherwise error code.
 */
EXTERNDOM OSRTDOMError domAddContent (OSRTDOMNodePtr node,
                                      const OSUTF8CHAR* pContent,
                                      size_t contentLen);

/**
 * Adds CDATA section to the node.
 *
 * @param node       Pointer to the node where content to be added.
 * @param pContent   Pointer to UTF-8 encoded content to be wrapped by
 *                     CDATA section.
 * @param contentLen Length of the content
 * @return   0 - success, otherwise error code.
 */
EXTERNDOM OSRTDOMError domAddCdata (OSRTDOMNodePtr node,
                                    const OSUTF8CHAR* pContent,
                                    size_t contentLen);

EXTERNDOM OSBOOL domIsContent (OSRTDOMNodePtr node);

#ifdef __cplusplus
}
#endif
/**
 * @}
 */
#endif /* __DOMAPI_H */

